#!/opt/anaconda1anaconda2anaconda3/bin/python
# -*- coding: utf-8 -*-

import sys
import argparse
import uuid
import json
import socket
from flask import Flask, Response
from flask import request, current_app, stream_with_context

DEBUG = False
TERMINATOR = '\n'

KOLA_HTTP_DEFAULT_HOST = '127.0.0.1'
KOLA_HTTP_DEFAULT_PORT = 5505

KOLA_SERVER_HOST = '127.0.0.1'
KOLA_SERVER_PORT = 5500

app = Flask(__name__)
app.config.from_object(__name__)

# id 2 socket
socket_map = dict()


def make_rsp(rsp):
    format_tpl = u'\n/' + u'-' * 80 + u'\n%s\n' + u'-' * 80 + u'/\n'

    if isinstance(rsp, dict):
        rsp = json.dumps(rsp, indent=4, ensure_ascii=False)

    return format_tpl % rsp

@app.route('/call/<remote_endpoint>')
def remote_call(remote_endpoint):
    """
    保持链接，有数据就返回
    """
    request_id = uuid.uuid1().hex

    send_dict = request.args.to_dict()
    send_dict['endpoint'] = remote_endpoint

    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        sock.connect((app.config['KOLA_SERVER_HOST'], app.config['KOLA_SERVER_PORT']))
    except Exception, e:
        return make_rsp('connect to kola server fail. e: %s' % e)

    try:
        sock.send(json.dumps(send_dict)+current_app.config['TERMINATOR'])
    except Exception, e:
        return make_rsp('send to kola server fail. e: %s' % e)

    socket_map[request_id] = sock
    f = sock.makefile('rw')

    def generate():
        yield make_rsp('request_id: %s' % request_id)

        while True:
            current_app.logger.debug('waiting data...')
            try:
                recv_data = f.readline()
            except Exception, e:
                yield make_rsp('readline raise exception: %s' % e)
                break

            if not recv_data:
                if request_id in socket_map:
                    yield make_rsp('server side closed socket')
                    socket_map.pop(request_id).close()
                else:
                    yield make_rsp('client side closed socket')
                # socket关闭，其makefile出来的file是不会自动关闭的
                f.close()
                break
            else:
                current_app.logger.debug('recv_data: %s' % recv_data)
                try:
                    jdata = json.loads(recv_data)
                    yield make_rsp(jdata)
                except Exception, e:
                    current_app.logger.error('json.loads(recv_data) fail: %s', e)
                    yield make_rsp(recv_data)

    return Response(stream_with_context(generate()))


@app.route('/close/<request_id>')
def close_conn(request_id):
    sock = socket_map.pop(request_id, None)

    if not sock:
        return make_rsp('sock not in socket_map')
    else:
        # 这个是真正破坏掉链接，close则不会
        try:
            sock.shutdown(socket.SHUT_RDWR)
        except:
            pass #some platforms may raise ENOTCONN here
        # 还要再调用一次close，参看 SocketServer
        sock.close()
        return make_rsp('sock succ closed')


@app.route('/stat')
def stat():
    stat_dict = dict([(k, v.fileno()) for k, v in socket_map.items()])
    return make_rsp(stat_dict)


def build_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--host', default=app.config['KOLA_HTTP_DEFAULT_HOST'], help='bind host', action='store')
    parser.add_argument('-p', '--port', default=app.config['KOLA_HTTP_DEFAULT_PORT'], type=int, help='bind port', action='store')
    parser.add_argument('-a', '--khost', default=app.config['KOLA_SERVER_HOST'], help='kola host', action='store')
    parser.add_argument('-b', '--kport', default=app.config['KOLA_SERVER_PORT'], type=int, help='kola port', action='store')
    parser.add_argument('-d', '--debug', default=False, help='debug mode', action='store_true')
    return parser


def main():
    # 得用gevent来执行，否则会被一个命令卡住
    from gevent import monkey; monkey.patch_all()
    from gevent import wsgi

    args = build_parser().parse_args()
    app.debug = args.debug
    app.config['KOLA_SERVER_HOST'] = args.khost
    app.config['KOLA_SERVER_PORT'] = args.kport

    app.logger.info("* Running kola_http on http://%s:%s, kola is %s:%s, debug:%s" % (
        args.host, args.port, args.khost, args.kport, args.debug
    ))
    try:
        wsgi.WSGIServer((args.host, int(args.port)), app).serve_forever()
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == '__main__':
    main()

